/**
* Copyright 2005 The Apache Software Foundation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
 
package org.apache.nutch.indexer.geo;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import org.apache.nutch.parse.Parse;

import org.apache.nutch.indexer.IndexingFilter;
import org.apache.nutch.indexer.IndexingException;
import org.apache.nutch.fetcher.FetcherOutput;
import org.apache.nutch.pagedb.FetchListEntry;

import org.apache.nutch.util.LogFormatter;
import org.apache.nutch.util.NutchConf;
import org.apache.nutch.util.PrefixStringMatcher;

import java.util.HashMap;
import java.util.regex.*;
import java.util.StringTokenizer;
import java.util.logging.Logger;
import java.lang.Float;
import java.io.*;

/** Adds geographic position fields to a document. */
public class GeoIndexingFilter implements IndexingFilter {

    public static NutchConf nutchConf = new NutchConf();
    
    public static final Logger LOG = LogFormatter.getLogger(GeoIndexingFilter.class.getName());

    private static final String DOMAIN2POSITIONFile =
        nutchConf.get("geoPosition.domain2position.file", "geodata.txt");

    HashMap h = new HashMap();
    PrefixStringMatcher prematch;

    private class PositionPolar {

        float north, east;

        /** Creates a new instance of IndexSegmentGeoObj */
        private PositionPolar(float pNorth,float pEast) {
            north = pNorth;
            east = pEast;
        }

        private float getNorth() {
            return north;
        }

        private float getEast() {
            return east;
        }
    }

    /**read in a file and split Urls and Position-Tags*/
    public GeoIndexingFilter() {

        LOG.info("Reading " + DOMAIN2POSITIONFile + " for indexing");
        try {
            Reader reader = nutchConf.getConfResourceAsReader(DOMAIN2POSITIONFile);

            BufferedReader  in = new BufferedReader(reader);

            //reading
            String s;
            while((s = in.readLine())!=null) {

                StringTokenizer split = new StringTokenizer(s,"\t");
                String url = split.nextToken();
                String northString = split.nextToken();
                String eastString = split.nextToken();

                float north = 0;
                float east = 0;

                try {
                    north = Float.parseFloat(northString);

                    } catch(NumberFormatException e) {
                        System.out.println(e);
                        return;
                }
                try {
                    east = Float.parseFloat(eastString);

                    } catch(NumberFormatException f) {
                        System.out.println(f);
                        return;
                }
               
                h.put(url, new GeoIndexingFilter.PositionPolar(north, east));
                
            }
            
        } catch (IOException e) {
            LOG.info(DOMAIN2POSITIONFile+" not found");
        }
        LOG.info("hashtable.size: "+h.size());
        LOG.info("Finished reading geo file: " + DOMAIN2POSITIONFile);

	prematch = new PrefixStringMatcher(h.keySet());
	LOG.info("Amount positions read in file: " + h.size());
    }

    /**set the Position-Tags */
    public Document filter (Document doc, Parse parse, FetcherOutput fo)
        throws IndexingException {
        //LOG.info("Filter wird gestartet!");
        String no = parse.getData().getMetadata().getProperty("North");
        String ea = parse.getData().getMetadata().getProperty("East");
        String el = parse.getData().getMetadata().getProperty("Elevation");



        if ( no != null && ea != null) {

        System.out.println("W1 ");

            try {
                float North = Float.parseFloat(no);
                float East = Float.parseFloat(ea);

                Domain2Position xyz = new Domain2Position(North, East);

                //add lucene Terms
                doc.add(Field.UnIndexed("north", xyz.formatPolar(North)));
                doc.add(Field.UnIndexed("east",  xyz.formatPolar(East)));
                doc.add(Field.UnStored("posX",   xyz.formatCartesian(xyz.getX())));
                doc.add(Field.UnStored("posY",   xyz.formatCartesian(xyz.getY())));
                doc.add(Field.UnStored("posZ",   xyz.formatCartesian(xyz.getZ())));

                if ( el != null) {
                    try {
                        float Elevation = Float.parseFloat(el);
                        doc.add(Field.Text("elevation", xyz.formatMeters(Elevation)));

                    } catch (NumberFormatException d) {
                        System.out.println(d);
                    }
                }

            } catch (NumberFormatException c) {
                System.out.println(c);
            }
            //LOG.info("found Position in Document " + fo.getUrl());


        } else {
            PositionPolar aPositionPolar;
            FetchListEntry fle = fo.getFetchListEntry();
            String url = fle.getPage().getURL().toString();

            if ((aPositionPolar = (PositionPolar)h.get(prematch.longestMatch(url))) != null) {

                LOG.info("found Position in Position File " + fo.getUrl());

                Domain2Position xyz = new Domain2Position (aPositionPolar.getNorth(),aPositionPolar.getEast());

                doc.add(Field.UnIndexed("north", xyz.formatPolar(aPositionPolar.getNorth())));
                doc.add(Field.UnIndexed("east",  xyz.formatPolar(aPositionPolar.getEast())));
                doc.add(Field.UnStored("posX",   xyz.formatCartesian(xyz.getX())));
                doc.add(Field.UnStored("posY",   xyz.formatCartesian(xyz.getY())));
                doc.add(Field.UnStored("posZ",   xyz.formatCartesian(xyz.getZ())));
            } else {
                //LOG.info("Nothing found for " + fo.getUrl());
            }
        }
        //LOG.info("Filter ende!!!!!!!!!!!!!!");
        return doc;
    }

	public void test(String url) {
        PositionPolar aPositionPolar;
       System.out.println("tst1");
	    if ((aPositionPolar = (PositionPolar)h.get(prematch.longestMatch(url))) != null) {
            System.out.println("tst2");
            Domain2Position xyz = new Domain2Position (0,0);
            System.out.println("tst3");
            System.out.println (xyz.formatPolar(aPositionPolar.getNorth()));  
            System.out.println (xyz.formatPolar(aPositionPolar.getEast()));
        
            xyz.setNorth(aPositionPolar.getNorth());
            xyz.setEast(aPositionPolar.getEast());
            System.out.println (xyz.formatCartesian(xyz.getX()));
            System.out.println (xyz.formatCartesian(xyz.getY()));
            System.out.println (xyz.formatCartesian(xyz.getZ()));
        }
    }
    
    public static void main(String[] args) throws Exception {
        /*if (args.length == 0) {
            System.out.println ("java org.apache.nutch.indexer.geo.GeoIndexingFilter <domain> XXX");
            System.exit(-1);
        }*/
           System.out.println("tst-1");
        GeoIndexingFilter filter = new GeoIndexingFilter();
           System.out.println("tst0");
        filter.test("http://www.stuttgart.de");
    }
}
