/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.nutch.parse.geo;

import org.apache.nutch.parse.*;
import org.apache.nutch.parse.HtmlParseFilter;
import org.apache.nutch.parse.ParseException;
import org.apache.nutch.protocol.Content;
import org.apache.nutch.util.LogFormatter;

import org.w3c.dom.*;

import java.util.logging.Logger;
import java.util.regex.*;
import java.util.Properties;

public class GeoParser implements HtmlParseFilter {
  public static final Logger LOG = LogFormatter.getLogger("org.apache.nutch.parse.geo");
    
  private static Pattern pattern = 
    Pattern.compile("\\s*([0-9\\.]+)\\s*[;,]\\s*([0-9\\.]+)" , Pattern.CASE_INSENSITIVE);
    
  private static Pattern patterndc = 
    pattern.compile("([a-z0-9]+)\\s*=\\s*([a-z0-9\\.]+)", Pattern.CASE_INSENSITIVE);

  private String type;  
  private String north; 
  private String east;
  private String elevation;
    
  private void setFields() {
   type = null;
   north = null;
   east = null;
   elevation = null;
  }
  
  private void getMetaDirectives(Node node) {
           
    if (node.getNodeType() == Node.ELEMENT_NODE) {

      if ("BODY".equals(node.getNodeName())) {
        // META tags should not be under body
        return;
      }

      if ("META".equalsIgnoreCase(node.getNodeName())) {
        
        NamedNodeMap attrs = node.getAttributes();
                                
        Node NameNode = attrs.getNamedItem("name");
        
        //check for Geo.Position Entry
        if ( (NameNode != null) 
             && ("geo.position".equalsIgnoreCase(NameNode.getNodeValue())) ) {          
            
          Node contentNode = attrs.getNamedItem("content");
          if (contentNode != null) {
     
              String content = contentNode.getNodeValue().toLowerCase();
              Matcher match = pattern.matcher(content);
              if (match.find()) {            
       
                  north = match.group(1);
                  east = match.group(2);
                  type = "Geo.Position";
            }
          }
        }   
        
        //check for ICBM Entry                
        if ( (NameNode != null) 
             && ("ICBM".equalsIgnoreCase(NameNode.getNodeValue())) ) {
         
          Node contentNode= attrs.getNamedItem("content");
          if (contentNode != null) {
              
             String content= contentNode.getNodeValue().toLowerCase();
             Matcher match = pattern.matcher(content);
             if (match.find()) {
                north = match.group(1);
                east = match.group(2);
                type = "ICBM";     
            }            
          }  
        }
        
        //check for DC.coverage.spatial Entry
        if ( (NameNode != null) 
             && ("DC.coverage.spatial".equalsIgnoreCase(NameNode.getNodeValue())) ) {
          
          Node schemeNode= attrs.getNamedItem("scheme");       
          
          if ((schemeNode != null) 
            && "DCMIPOINT".equalsIgnoreCase(schemeNode.getNodeValue())) {          
          Node contentNode= attrs.getNamedItem("content");

          if (contentNode != null) {             
             String content= contentNode.getNodeValue().toLowerCase();
             Matcher match = patterndc.matcher(content);
                           
             while (match.find()) {                    
                    
                 if (match.group(1).equals("north")) {                    
                     north = match.group(2);                     
                     
                 } else if (match.group(1).equals("east")) {
                      east = match.group(2);                    
                      
                 } else if (match.group(1).equals("elevation")) {
                    elevation = match.group(2);                    
                 }               
             }                    
             type = "DC.coverage.spatial";         
            
            }
          }
        }
      }
    
    }// if get.NodeType    
        NodeList children = node.getChildNodes();
        if ( children != null) {
          int len = children.getLength();
          for ( int i = 0; i < len; i++ ) {
            getMetaDirectives(children.item(i));
          }
    }
  }//end of getMetaDirectives
  
  private Properties geometa;
  
  /*filter init by HtmlParseFilter*/
  public Parse filter(Content content, Parse parse, HTMLMetaTags metaTags, DocumentFragment doc) {
                   
        setFields();                  
                   
        getMetaDirectives(doc);               
    
        geometa = new Properties();
       
        if (north != null && east != null) {
        
            geometa.put("Geodata",type);
            geometa.put("North", north);
            geometa.put("East", east);
            
            if (elevation != null) {
                geometa.put("Elevation", elevation);
            }
            
            //write in parse for Indexing          
            parse.getData().getMetadata().putAll(geometa);
            LOG.info("Found Geodata in: "+content.getUrl());
     }
          
    return parse;
  }
  
}
