/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.nutch.searcher.geo;

import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.RangeQuery;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.*;
import org.apache.lucene.index.Term;

import org.apache.nutch.searcher.QueryFilter;
import org.apache.nutch.searcher.QueryException;
import org.apache.nutch.searcher.Query;
import org.apache.nutch.searcher.Query.*;
import org.apache.nutch.searcher.Query.Clause;

import org.apache.nutch.indexer.geo.Domain2Position;


import org.apache.nutch.util.NutchConf;
import org.apache.nutch.util.LogFormatter;

import java.util.*;
import java.util.regex.*;
import java.util.logging.Logger;
import java.net.URL;
import java.io.*;


/** Handles "position:" query clauses, causing them to search the field indexed by
 * GeoIndexingFilter and zips. */
public class GeoQueryFilter implements QueryFilter {
    
    public static NutchConf nutchConf = new NutchConf();
    public static final Logger LOG = LogFormatter.getLogger(GeoQueryFilter.class.getName());
    
    //directory where the zip-code file are
    public static final String ZIPDIR = nutchConf.get("geoposition.zips.dir","zip/");
    
    //ISO 3166 Codes (Countries) which are used
    public static final String ZIPUSE = nutchConf.get("geoposition.zips.use");
    
    HashMap h = new HashMap();
    
    //made of ZIPUSE
    private Vector use = new Vector();
    
    
    public GeoQueryFilter(){
              
       StringTokenizer stt = new StringTokenizer(ZIPUSE,";");
       String sUSE = new String();
       
       while(stt.hasMoreTokens()){
           sUSE = stt.nextToken();
           //skip empty tokens
           if(!"".equals(sUSE)){ 
            use.add(sUSE);
          }
       }
       
       //reads all files in the ZIPDIR-derectory which are in ZIPUSE
       for(int j=0; j<use.size(); j++){
       
           StringBuffer buffer = new StringBuffer(ZIPDIR+use.elementAt(j).toString()+".geo.txt");    
           String file = buffer.toString();

           Reader reader = nutchConf.getConfResourceAsReader(file);

           BufferedReader in = new BufferedReader(reader);

           String s;
           try{
           
               while(( s= in.readLine()) != null) {
                   StringTokenizer split = new StringTokenizer(s,"\t");
                   String zipcode = split.nextToken();
                   String northString = split.nextToken();
                   String eastString = split.nextToken();

                   float north = 0;
                   float east = 0;

                   try {
                       north = Float.parseFloat(northString);

                   } catch(NumberFormatException e) {
                       System.out.println(e);
                       return;
                   }
                   try {
                       east = Float.parseFloat(eastString);

                       } catch(NumberFormatException f) {
                       System.out.println(f);
                       return;
                   }
                   StringBuffer sb = new StringBuffer(use.elementAt(j)+zipcode);
                   zipcode = sb.toString();
                   
                   //hashmap is filled with the infomation in the zip-codes files
                   h.put(zipcode, new GeoQueryFilter.PositionPolar(north, east));
               }
           }
           catch(Exception ex){
               ex.printStackTrace();
           }
       }
    } 
    
   public BooleanQuery filter(Query input, BooleanQuery output)
       throws QueryException {
        
       // examine each clause in the Nutch query
       Clause[] clauses = input.getClauses();
    
       for (int i = 0; i <clauses.length; i++) { 
              
           Clause c = clauses[i];
                   
           float north = 0;
           float east = 0;
           //default value: radius=25km
           int radius = 25;
       
           if (c.getField().equals("position")) {
               
               String position = c.getTerm().toString().trim().toLowerCase();

               //examine the coordnates from position
               Pattern p = Pattern.compile("[nwesr][\\d\\.-]+");
               Matcher m = p.matcher(position);

               //identify strings in the field
               while (m.find()) {
                   if(m.group().startsWith("n")) {
                       try {
                           north = Float.parseFloat(m.group().replaceFirst("n",""));
                       } catch (NumberFormatException e) {
                           System.out.println(e);
                       }
                   } else if (m.group().startsWith("e")) {
                       try{
                           east = Float.parseFloat(m.group().replaceFirst("e",""));
                       } catch (NumberFormatException f) {
                           System.out.println(f);
                       }
                   } else if (m.group().startsWith("s")) {
                       try {
                           north = -1 * Float.parseFloat(m.group().replaceFirst("s",""));
                       } catch (NumberFormatException e) {
                           System.out.println(e);
                       }
                   } else if (m.group().startsWith("w")) {
                       try {
                           east = -1 * Float.parseFloat(m.group().replaceFirst("w",""));
                       } catch (NumberFormatException e) {
                           System.out.println(e);
                       } 
                   } else if (m.group().startsWith("r")) {
                       try {
                           radius = Integer.parseInt(m.group().replaceFirst("r",""));
                       } catch (NumberFormatException f) {
                           System.out.println(f);
                       }
                   }
               }
               LOG.info("Searching position: north:"+north+" east: "+east);
           }
       
           //proofs each tld(iso-code) of the use-vector
           Enumeration e = use.elements();
           String tld;
           while(e.hasMoreElements()){
               tld=(String)e.nextElement();
               if(c.getField().equals(tld)){

                   String zip = c.getTerm().toString().trim().toLowerCase();
                   
                   LOG.info("Term: "+zip);
                   
                   StringTokenizer tokenizer = new StringTokenizer(zip,"r");
                   zip = tokenizer.nextToken();
                   String rad = new String();
                   if(tokenizer.hasMoreElements()){
                       try{
                            rad = tokenizer.nextToken();
                            radius = Integer.parseInt(rad);
                       }
                       catch(Exception ex){
                           //if parsing radius from the query fails, radius becomes default value
                           radius = 25;
                       }
                   }
                   LOG.info("ZIP: "+zip);
                   LOG.info("Radius: "+radius);
                   
                   StringBuffer tldbuffer = new StringBuffer(tld+zip);
                   zip=tldbuffer.toString();
                   
                   LOG.info("Searching for zip: "+zip);

                   PositionPolar aPositionPolar = (PositionPolar)h.get(zip);

                   //if the zip is in the zip-code files
                   if(aPositionPolar!=null) {
                        north = aPositionPolar.getNorth();
                        east = aPositionPolar.getEast();
                   }
                   break;
               }
           }

           //skip non-position and non-zip files
           if(north !=0) {
               //Calculates X,Y,Z and returns lucene RangeQuerys
               Domain2Position geo = new Domain2Position(0,0);
               geo.setNorth(north);
               geo.setEast(east);     
               radius *= 1000 / geo.STEP; 

               //calculate X Range and add luceneTerm
               Term posXmin = new Term("posX", geo.formatCartesian(geo.getX() - radius));
               Term posXmax = new Term("posX", geo.formatCartesian(geo.getX() + radius));  
               output.add(new RangeQuery(posXmin,posXmax,true), c.isRequired(), c.isProhibited());

               //calc Y Range and add luceneTerm
               Term posYmin = new Term("posY", geo.formatCartesian(geo.getY() - radius));
               Term posYmax = new Term("posY", geo.formatCartesian(geo.getY() + radius));  
               output.add(new RangeQuery(posYmin,posYmax,true), c.isRequired(), c.isProhibited());

               //calc Z Range and add luceneTerm
               Term posZmin = new Term("posZ", geo.formatCartesian(geo.getZ() - radius));
               Term posZmax = new Term("posZ", geo.formatCartesian(geo.getZ() + radius));  
               output.add(new RangeQuery(posZmin,posZmax,true), c.isRequired(), c.isProhibited());
           }
       }   
       return output;
   }//end filter
    
   //Class to store the Coordinates of the zipcode
   private class PositionPolar {

       float north, east;

       /** Creates a new instance of IndexSegmentGeoObj */
       private PositionPolar(float pNorth,float pEast) {
           north = pNorth;
           east = pEast;
       }

       private float getNorth() {
           return north;
       }

       private float getEast() {
           return east;
       }
   }
}
